@@ -26,6 +26,7 @@ Pixmap icon, logo;
 extern Widget toplevel, base, menubar, field, info;
 extern Widget scorebox, highscorebox;
 extern Widget endgamebox, enternamebox;
+extern Widget pausebutton;
 int in_window, level, score;
 int efficiency;
 XtIntervalId timer;
@@ -134,8 +135,6 @@ int move_bill (monster *guy, horde *bill, int level, int mode) {
 	int sx = (abs(xdist) == xdist) * 2 - 1;
 	int sy = (abs(ydist) == ydist) * 2 - 1;
 	if (!xdist && !ydist) return 0;
-	else if (!xdist) guy->y+=step;
-	else if (!ydist) guy->x+=step;
 	else if ( abs(xdist)<step && abs(ydist)<step ) {
 		guy->x = guy->target_x;
 		guy->y = guy->target_y;
@@ -444,26 +443,6 @@ void update_info(Widget info, horde *bill, network *net, library *systems,
 	efficiency += temp;
 }
   
-void update_high_scores() {
-	char str[1000], temp[40];
-	int i, j;
-	strcpy (str,"               High Scores:          \n\n");
-	strcat (str, "Name                     Level     Score\n");
-	for (i=0; i<10; i++) {
-		strcat (str, scores[i].name);
-		for (j=strlen(scores[i].name); j<25; j++) strcat (str, " ");
-		sprintf (temp, "%5d   %7d\n", scores[i].level, scores[i].score);
-		strcat (str, temp);
-	}
-	print_to_widget(XmMessageBoxGetChild(highscorebox, XmDIALOG_MESSAGE_LABEL), str);
-}
-
-void update_scorebox() {
-	char str[40];
-	sprintf (str, "After Level %d:\nYour score: %d", level, score);
-	print_to_widget(XmMessageBoxGetChild(scorebox, XmDIALOG_MESSAGE_LABEL), str);
-}
-
 void update_score (horde *bill, network *net, int level, int action) {
 	switch (action){
 		case ENDLEVEL: score+=(level*efficiency/iteration); break;
@@ -543,9 +522,16 @@ void write_high_scores(scorelist *scores) {
 /*  Warp to specified level   */
 void warp_to_level (int lev) {
 	XDefineCursor(display, window, defaultcursor);
-	if (lev <= level) return;
-	level = lev;
-	setup_level(&bill, &net, &systems, level);
+	if (state==PLAYING) {
+		if (lev <= level) return;
+		level = lev;
+		setup_level(&bill, &net, &systems, level);
+	}
+	else {
+		if (lev<=0) return;
+		new_game ((Widget)0, (XtPointer)NULL, (XtPointer)NULL);
+		level = lev;
+	}
 }
 
 /*  Add new high score to list   */
@@ -573,6 +559,7 @@ void new_game (Widget w, XtPointer client_data, XtPointer call_data) {
 	score = 0;
 	XtRemoveTimeOut(timer);
 	timer = XtAppAddTimeOut (app, INTERVAL, update_all, NULL);
+	XtSetSensitive(pausebutton, True);
 }
 
 /*  Quit game callback function*/
@@ -582,17 +569,18 @@ void quit_game (Widget w, XtPointer client_data, XtPointer call_data) {
 }
 
 void popdown (Widget w, XtPointer client_data, XtPointer call_data) {
-	in_popup=1;
+	in_popup=0;
+	XtSetSensitive(menubar, True);
 }
 
 void popup (Widget w, Widget *box, XtPointer call_data) {
 	XtManageChild(*box);
-	XtAddCallback(XtParent(*box), XmNpopdownCallback, (XtCallbackProc) popdown, NULL);
+	XtAddCallback(XtParent(*box), XtNpopdownCallback, (XtCallbackProc) popdown, NULL);
+	in_popup=1;
+	XtSetSensitive(menubar, False);
 	XtPopup(XtParent(*box), XtGrabNone);
-	XWarpPointer(display, 0, XtWindow(*box), 0, 0, 0, 0, 10, 10);
-	while (in_popup==0 || XtAppPending(app))
+	while (in_popup==1 || XtAppPending(app))
 		XtAppProcessEvent(app, XtIMAll);
-	in_popup=0;
 }
 
 /*  Helper function for event handling of hits */
@@ -697,6 +685,18 @@ void redraw_window(Widget w, XtPointer client_data, XEvent *event) {
 	refresh_field(field);
 }
 
+/* Event handler (key hit - debugging) */
+void print_debug_info(Widget w, XtPointer client_data, XKeyEvent *event) {
+	node *temp;
+	if (event->keycode != XKeysymToKeycode(display, 'd')) return;
+	printf ("Bills going in:\n");
+	for (temp=bill.in; temp; temp=temp->next) printf ("\t%d, %d\n", temp->data->x, temp->data->y);
+	printf ("Bills at computers :\n");
+	for (temp=bill.at; temp; temp=temp->next) printf ("\t%d, %d\n", temp->data->x, temp->data->y);
+	printf ("Bills going out:\n");
+	for (temp=bill.out; temp; temp=temp->next) printf ("\t%d, %d\n", temp->data->x, temp->data->y);
+}
+
 void update_all(XtPointer client_data, XtIntervalId *timer_id) {
 	int n;
 	timer = XtAppAddTimeOut (app, INTERVAL, update_all, NULL);
@@ -730,6 +730,7 @@ void update_all(XtPointer client_data, XtIntervalId *timer_id) {
 		case END:
 			XtRemoveTimeOut(timer);
 			timer = XtAppAddTimeOut (app, INTERVAL*1000, NULL, NULL);
+			XtSetSensitive(pausebutton, False);
 			convert_to_toasters(&net);
 			update_network(&net);
 			refresh_field(field);
@@ -737,7 +738,7 @@ void update_all(XtPointer client_data, XtIntervalId *timer_id) {
 			if (score > scores[9].score) {
 				popup(NULL, &enternamebox, NULL);
 			}
-			update_high_scores();
+			update_high_scores(highscorebox, scores);
 			popup(NULL, &highscorebox, NULL);
 			sleep(1);
 			clean_offscreen (values.foreground, values.background); 
@@ -746,7 +747,7 @@ void update_all(XtPointer client_data, XtIntervalId *timer_id) {
 				(height-LOGO_HEIGHT)/2);
 			break;
 		case BETWEEN:
-			update_scorebox();
+			update_scorebox(scorebox, level, score);
 			popup (NULL, &scorebox, NULL);
 			state = PLAYING;
 			setup_level(&bill, &net, &systems, ++level);
@@ -850,7 +851,7 @@ main(int argc, char **argv) {
 		LOGO_HEIGHT, (width-LOGO_WIDTH)/2, (height-LOGO_HEIGHT)/2);
 	refresh_field(field);
 	load_icon(&icon, attr);
-	XtVaSetValues (toplevel, XmNiconPixmap, icon, NULL);
+	XtVaSetValues (toplevel, XtNiconPixmap, icon, NULL);
 	DPRINTF("Hey, we loaded the icon\n");
 
 /**********************************/
@@ -861,7 +862,7 @@ main(int argc, char **argv) {
 	setup_other_widgets(about, rules, thanks, story, freeware);
 	DPRINTF("Hey, we set up the other widgets\n");
 	read_high_scores(scores);
-	update_high_scores();
+	update_high_scores(highscorebox, scores);
 	DPRINTF("Hey, we got the high scores\n");
 	
 /*****************************/	
@@ -912,6 +913,7 @@ main(int argc, char **argv) {
 		state = PLAYING;
 	}
 	else timer = XtAppAddTimeOut (app, INTERVAL*1000, NULL, NULL);
+	XtSetSensitive(pausebutton, False);
 	DPRINTF("Hey, we're about to start...\n");
 
 /**********   Main Loop  ***********/
